@@ -2,13 +2,12 @@
 #' @importFrom stats runif optim 
 bestConvex <- function(y, experts, awake = NULL, loss.type = list(name = "square"), 
   niter = 1, ...) {
-  experts <- matrix(as.numeric(as.matrix(experts)), nrow = length(y))
   N <- ncol(experts)
   
   # if there are no NA and if awake is null we can perform an exact resolution for
   # the square loss
   idx.na <- which(is.na(experts))
-  if (length(idx.na) == 0 && is.null(awake) && loss.type$name == "square") {
+  if (length(idx.na) == 0 && is.null(awake) && is.list(loss.type) && ! is.null(loss.type$name) && loss.type$name == "square") {
     y.na <- is.na(y)
     y <- y[!y.na]
     x <- experts[!y.na, ]
@@ -33,13 +32,13 @@ bestConvex <- function(y, experts, awake = NULL, loss.type = list(name = "square
     if (!is.null(res)) {
       coefficients <- matrix(res$solution, ncol = N)
       prediction <- experts %*% t(coefficients)
-      bestLoss <- mean(loss(x = prediction, y))
+      bestLoss <- mean(loss(x = prediction, y = y))
     }
   } else {
     res <- NULL
   }
   if (is.null(res)) {
-    warning("The best convex oracle is only approximated (using optim).")
+    # warning("The best convex oracle is only approximated (using optim).")
     if (is.null(awake)) {
       awake <- as.matrix(array(1, dim(experts)))
     }
@@ -57,7 +56,6 @@ bestConvex <- function(y, experts, awake = NULL, loss.type = list(name = "square
       # Random initialization
       p <- runif(N, 0, 1)
       p <- p/sum(p)
-      
       # Convex optimization
       w <- optim(p, lossp, gr = NULL, lower = 1e-20, method = "L-BFGS-B", ...)
       
@@ -70,7 +68,7 @@ bestConvex <- function(y, experts, awake = NULL, loss.type = list(name = "square
       }
     }
     coefficients <- matrix(best_p, ncol = N)
-    coefficients <- coefficients/apply(coefficients, 1, sum)
+    coefficients <- coefficients/rowSums(coefficients)
     pond <- awake %*% t(coefficients)
     prediction <- ((as.numeric(experts) * awake) %*% t(coefficients))/pond
   }
