@@ -3,13 +3,13 @@ print.oracle <- function(x, ...) {
   cat("Call:\n")
   print(x$call)
   
-  if (x$d > 1) {
+  if (length(dim(x$experts)) > 2) {
     x$experts <- blockToSeries(x$experts)
+    x$Y <- blockToSeries(x$Y)
   }
   if (x$model != "shifting") {
     cat("\nCoefficients:\n")
-    x$coefficients <- data.frame(matrix(as.numeric(as.matrix(x$coefficients)), 
-                                        nrow = 1))
+    x$coefficients <- data.frame(as.list(x$coefficients))
     names(x$coefficients) <- colnames(x$experts)
     rownames(x$coefficients) <- ""
     print(signif(x$coefficients, digits = 3))
@@ -23,7 +23,7 @@ print.oracle <- function(x, ...) {
 #' @export
 summary.oracle <- function(object, ...) {
   
-  if (object$d > 1) {
+  if (length(dim(object$experts)) > 2) {
     object$experts <- blockToSeries(object$experts)
     object$Y <- blockToSeries(object$Y)
   }
@@ -33,11 +33,11 @@ summary.oracle <- function(object, ...) {
     T <- length(object$Y)
     K <- ncol(object$experts)
     
-    rmse.algo <- sqrt(mean(loss(object$prediction, object$Y)))
-    mape.algo <- mean(loss(object$prediction, object$Y, loss.type = "percentage"))
+    rmse.algo <- sqrt(mean(loss(x = object$prediction, y = object$Y)))
+    mape.algo <- mean(loss(x = object$prediction, y = object$Y, loss.type = list(name = "percentage")))
     rmse.unif <- sqrt(lossConv(rep(1/K, K), object$Y, object$experts, awake = object$awake))
     mape.unif <- lossConv(rep(1/K, K), object$Y, object$experts, awake = object$awake, 
-                          loss.type = "percentage")
+                          loss.type = list(name = "percentage"))
     
     
     TAB <- data.frame(rmse = c(rmse.algo, rmse.unif), mape = c(mape.algo, mape.unif))
@@ -49,8 +49,8 @@ summary.oracle <- function(object, ...) {
     x <- summary(oracle(object$Y, object$experts, model = "expert", loss.type = object$loss.type, 
                  awake = object$awake))
     
-    rmse.algo <- sqrt(mean(loss(object$prediction, object$Y)))
-    mape.algo <- mean(loss(object$prediction, object$Y, loss.type = "percentage"))
+    rmse.algo <- sqrt(mean(loss(x = object$prediction, y = object$Y)))
+    mape.algo <- mean(loss(x = object$prediction, y = object$Y, loss.type = list(name = "percentage")))
     
     TAB.lin <- data.frame(rmse = rmse.algo, mape = mape.algo)
     rownames(TAB.lin) <- paste("Best", object$model, "oracle: ")
@@ -64,15 +64,19 @@ summary.oracle <- function(object, ...) {
     n.shifts <- round(quantile(1:K))
     TAB <- matrix(object$loss[n.shifts], nrow = 1)
     colnames(TAB) <- paste(n.shifts - 1, "shifts")
-    rownames(TAB) <- paste("Average", object$loss.type, "loss:")
+    rownames(TAB) <- ifelse(is.function(object$loss.type), 
+                            paste0("Average loss (", attributes(object$loss.type)$srcref, "):"), 
+                            paste0("Average ", object$loss.type, "loss:"))
     
-    if (object$loss.type == "square") {
+    if (! is.function(object$loss.type) && object$loss.type == "square") {
       TAB <- sqrt(TAB)
       rownames(TAB) <- "rmse:"
-    } else if (object$loss.type == "absolute") {
+    } else if (! is.function(object$loss.type) && object$loss.type == "absolute") {
       rownames(TAB) <- "mean absolute error:"
-    } else if (object$loss.type == "percentage") {
+    } else if (! is.function(object$loss.type) && object$loss.type == "percentage") {
       rownames(TAB) <- "mape:"
+    } else {
+      rownames(TAB) <- "custom loss:"
     }
   }
   
