@@ -22,15 +22,12 @@
 #' sequence of experts with at most $m$ shifts that would have performed the
 #' best to predict the sequence of observations in \code{Y}.}
 #' }
-#' @param loss.type A string or a list with a component 'name' specifying
-#' the loss function considered to evaluate the performance. It can be
-#' 'square', 'absolute', 'percentage', or 'pinball'. In the case of the pinball loss, the quantile 
-#' can be provided by assigning to loss.type a list of two elements: 
+#' @param loss.type \code{character, list or function}. 
 #' \describe{
-#'      \item{name}{A string defining the name of the loss function (i.e., 'pinball')}
-#'      \item{tau}{ A number in \code{[0,1]} defining the quantile to be predicted. The default value is 0.5 to predict the median.}
-#' } 
-#' 
+#'      \item{character}{ Name of the loss to be applied ('square', 'absolute', 'percentage', or 'pinball');}
+#'      \item{list}{ When using pinball loss: list with field name equal to 'pinball' and field tau equal to the required quantile in [0,1];}
+#'      \item{function}{ A custom loss as a function of two parameters.}
+#' }
 #' @param lambda A positive number used by the 'linear' oracle only. 
 #' A possible $L_2$ regularization parameter for computing the linear oracle 
 #' (if the design matrix is not identifiable)
@@ -79,12 +76,16 @@ oracle <- function(Y, experts, model = "convex", loss.type = "square", awake = N
 oracle.default <- function(Y, experts, model = "convex", loss.type = "square", awake = NULL, 
   lambda = NULL, niter = NULL, ...) {
   
+  # checks
+  experts <- check_matrix(experts, "experts")
+  awake <- check_matrix(awake, "awake")
+  loss.type <- check_loss(loss.type = loss.type, loss.gradient = FALSE, model = model, use_cpp = FALSE)
+  
   # Test that Y and experts have correct dimensions
   if (is.null(Y) || is.null(experts)) {
     stop("Y and experts should not be null")
   }
   if (length(Y) == 1) {
-    experts <- as.matrix(experts)
     if (nrow(experts) == 1 || ncol(experts) == 1) {
       experts <- matrix(experts, nrow = 1)
     } else {
@@ -107,19 +108,6 @@ oracle.default <- function(Y, experts, model = "convex", loss.type = "square", a
   if (!(length(Y) == nrow(experts))) {
     stop("Bad dimensions: length(Y) should be equal to nrow(experts)")
   }
-  
-  if (is.null(loss.type)) {
-    loss.type <- list(name = "square")
-  }
-  if (!is.list(loss.type)) {
-    loss.type <- list(name = loss.type)
-  }
-  if (!(loss.type$name %in% c("pinball", "square", "percentage", "absolute"))) {
-    stop("loss.type should be one of these: 'absolute', 'percentage', 'square', 'pinball'")
-  }
-  if (!is.null(loss.type$tau) && loss.type$name != "pinball") {
-    warning("Unused parameter tau (loss.type != 'pinball')")
-  }
   if (!is.null(lambda) && model != "linear") {
     warning("Unused lambda parameter (model != 'linear')")
   }
@@ -146,11 +134,7 @@ oracle.default <- function(Y, experts, model = "convex", loss.type = "square", a
   if (!(model %in% c("convex", "linear", "shifting", "expert"))) {
     stop("Wrong model specification")
   }
-  if (min(Y) <= 0 && loss.type$name == "percentage") {
-    stop("Y should be non-negative for percentage loss function")
-  }
   names.experts <- colnames(experts)
-  experts <- matrix(as.numeric(as.matrix(experts)), nrow = length(Y))
   colnames(experts) <- names.experts
   
   # if we are looking for the best convex combination of experts
@@ -172,16 +156,16 @@ oracle.default <- function(Y, experts, model = "convex", loss.type = "square", a
   }
   
   if (!is.null(awake)) {
-    pond <- apply(awake,1,mean)
-    pred.unif <- apply(experts * awake, 1,mean) /pond
+    pond <- rowMeans(awake)
+    pred.unif <- rowMeans(experts * awake) / pond
     experts.pred <- experts * awake + pred.unif * (1-awake)
   } else {
     experts.pred <- experts
   }
   
-  loss.experts <- apply(apply(experts.pred, 2, function(x) {
-    loss(x, Y, loss.type = loss.type)
-  }), 2, mean)
+  loss.experts <- colMeans(apply(experts.pred, 2, function(x) {
+    loss(x = x, y = Y, loss.type = loss.type)
+  }))
   
   if (model == "expert") {
     best.loss <- min(loss.experts)
@@ -198,8 +182,8 @@ oracle.default <- function(Y, experts, model = "convex", loss.type = "square", a
   res$call <- match.call()
   if (model != "shifting") {
     res$residuals <- Y - res$prediction
-    res$loss <- mean(loss(res$prediction, Y, loss.type))
-    res$rmse <- sqrt(mean(loss(res$prediction, Y, "square")))
+    res$loss <- mean(loss(x = res$prediction, y = Y, loss.type = loss.type))
+    res$rmse <- sqrt(mean(loss(x = res$prediction, y = Y, loss.type = list(name = "square"))))
   }
   
   # we convert the data back to d-dimensional series if needed
