@@ -1,12 +1,15 @@
 #' Print an aggregation procedure
-#' @describeIn mixture \code{print}
+#' 
 #' @param x An object of class mixture
+#' 
 #' @export 
+#' 
+#' @rdname mixture-opera
 print.mixture <- function(x, ...) {
   cat("Aggregation rule: ")
   cat(x$model, "\n")
-  cat("Loss function: ", x$loss.type$name, "loss", "\n")
-  cat("Gradient trick: ", x$loss.gradient, "\n")
+  cat("Loss function: ", ifelse(is.function(x$loss.type), as.character(attributes(x$loss.type)[[1]]), paste0(x$loss.type$name, "loss")), "\n")
+  cat("Gradient trick: ", if (! is.null(x$loss.gradient) && is.function(x$loss.gradient)) {as.character(attributes(x$loss.gradient)[[1]])} else {x$loss.gradient}, "\n")
   cat("Coefficients: ")
   if (x$coefficients[1] != "Uniform") {
     cat("\n")
@@ -17,4 +20,4 @@ print.mixture <- function(x, ...) {
   } else {
     print("Uniform mixture")
   }
-}
\ No newline at end of file
+}
